@@ -3,8 +3,9 @@ import os
 import signal
 import sys
 from abc import ABC
+from enum import Enum
 from pathlib import Path
-from typing import Callable, List, Optional
+from typing import Callable, ClassVar, List, Optional
 
 from jsonschema.exceptions import ValidationError as JsonSchemaValidationError
 from requests.exceptions import RequestException
@@ -193,14 +194,23 @@ class FieldReadException(CliBaseException):
 
     """Error reading field(s) from remote state."""
 
-    def __init__(self, company_slug: str, dataset_name: str):
-        super().__init__(f'Error fetching field for company {company_slug} and dataset {dataset_name}')
+    def __init__(self, company_slug: str, dataset_name: Optional[str]):
+        dataset_message = f' under dataset {dataset_name} ' if dataset_name else ''
+        super().__init__(f'Error fetching field for company {company_slug}{dataset_message}')
+
+
+class ValidationErrorSeverity(Enum):
+
+    WARNING = 'WARNING'
+    ERROR = 'ERROR'
 
 
 class ValidationError(CliBaseException, ABC):
 
     """Abstract error raised during validation step."""
 
+    severity: ClassVar[ValidationErrorSeverity] = ValidationErrorSeverity.ERROR
+
 
 class FileMissingError(ValidationError):
 
@@ -217,6 +227,12 @@ class FileMissingError(ValidationError):
 
         super().__init__(msg)
 
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, FileMissingError):
+            return False
+
+        return str(self) == str(o)
+
 
 class DuplicateModelNameError(ValidationError):
 
@@ -231,6 +247,12 @@ class DuplicateModelNameError(ValidationError):
         path_lines = ''.join(f'\n  in {path}' for path in paths)
         super().__init__(f'Multiple model files use model name {model_name}{path_lines}')
 
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, DuplicateModelNameError):
+            return False
+
+        return str(self) == str(o)
+
 
 class DuplicateFieldSlugError(ValidationError):
 
@@ -246,6 +268,12 @@ class DuplicateFieldSlugError(ValidationError):
         dataset_message = f' under dataset {dataset_slug} ' if dataset_slug else ''
         super().__init__(f'Multiple field files{dataset_message}use slug {field_slug}{path_lines}')
 
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, DuplicateFieldSlugError):
+            return False
+
+        return str(self) == str(o)
+
 
 class InvalidYamlFile(ValidationError):
 
@@ -259,6 +287,31 @@ class InvalidYamlFile(ValidationError):
 
         super().__init__(f'Invalid YAML file - {error.problem}\n  on line {error.problem_mark.line}\n  in {path}')
 
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, InvalidYamlFile):
+            return False
+
+        return str(self) == str(o)
+
+
+class DeprecatedAttributeWarning(ValidationError):
+
+    severity = ValidationErrorSeverity.WARNING
+
+    def __init__(self, *, attribute: str, path: Path):
+        try:
+            path = path.relative_to(Path.cwd())
+        except ValueError:
+            pass  # Use relative path when possible
+
+        super().__init__(f'Deprecated attribute "{attribute}" \n  in {path}')
+
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, DeprecatedAttributeWarning):
+            return False
+
+        return str(self) == str(o)
+
 
 class JsonSchemaError(ValidationError):
     def __init__(self, *, path: Path, error: JsonSchemaValidationError):
@@ -269,6 +322,28 @@ class JsonSchemaError(ValidationError):
 
         super().__init__(f'{error.message}\n  in {path}')
 
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, JsonSchemaError):
+            return False
+
+        return str(self) == str(o)
+
+
+class MissingFieldFileError(ValidationError):
+    def __init__(
+        self,
+        *,
+        field_slug: str,
+        dataset_slug: Optional[str],
+    ) -> None:
+        super().__init__(f'Missing field file for slug {field_slug} under dataset {dataset_slug}')
+
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, MissingFieldFileError):
+            return False
+
+        return str(self) == str(o)
+
 
 def handle_exception(f: Callable):
     """Print exception and exit with error code."""
