@@ -68,12 +68,24 @@ class FileWriter:
         logger.debug(f'About to create an empty model {model_name}')
         path.touch()
 
+    def write_empty_field(self, field_slug: str):
+        """Create an empty field file."""
+        path = Paths.scanned_fields_dir() / f'{field_slug}{FileExtension.FIELD_YAML.value}'
+        logger.debug(f'About to create an empty field {field_slug}')
+        path.touch()
+
     def write_scanned_model(self, model: PanoModel):
         """Write scanned model to local filesystem."""
         path = Paths.scanned_dir() / f'{model.model_name}{FileExtension.MODEL_YAML.value}'
         logger.debug(f'About to write model {model.id}')
         write_yaml(path, model.to_dict())
 
+    def write_scanned_field(self, field: PanoField):
+        """"Write scanned field to local filesystem."""
+        path = Paths.scanned_fields_dir() / f'{field.slug}{FileExtension.MODEL_YAML.value}'
+        logger.debug(f'About to write field {field.slug}')
+        write_yaml(path, field.to_dict())
+
     def write_model(self, model: PanoModel, *, package: Optional[str] = None, file_name: Optional[str] = None):
         """Write model to local filesystem."""
         # Default to name of slugified name of DS
@@ -99,9 +111,9 @@ class FileWriter:
             file_name = f'{field.slug}{FileExtension.FIELD_YAML.value}'
 
         package = package if package is not None else field.data_source
-        if package:
+        if package is not None:
             # dataset-scope field
-            path = Paths.fields_dir(self.cwd / field.data_source) / file_name
+            path = Paths.fields_dir(self.cwd / package) / file_name
         else:
             # company-scope field
             path = Paths.fields_dir(self.cwd) / file_name
@@ -113,7 +125,7 @@ class FileWriter:
         """Delete field from local filesystem."""
         assert field.file_name is not None
 
-        if field.data_source:
+        if field.data_source is not None:
             # dataset-scope field
             path = Paths.fields_dir(self.cwd / field.data_source) / field.file_name
         else:
